package classes;

import managers.ConfigManager;
import enums.RoomSizeType;
import enums.RoomType;
import flixel.FlxG;
import managers.MapManager;
import managers.RoomManager;
import managers.StateManager;

class BuildRoomFrame extends AtlasSprite
{
	var roomType:RoomType;

	public function new(roomType:RoomType)
	{
		super();
		this.roomType = roomType;
		var roomSize:RoomSizeType = RoomManager.getRoomSizefromKind(roomType);

		switch (roomSize)
		{
			case RoomSizeType.None:
				return;
			case RoomSizeType.S1x1:
				assignImageFromAtlas(ConfigManager.BUILDF1_GFX);
			case RoomSizeType.S2x1:
				assignImageFromAtlas(ConfigManager.BUILDF2_GFX);
			case RoomSizeType.S4x1:
				assignImageFromAtlas(ConfigManager.BUILDF4_GFX);
		}
	}

	override public function update(elapsed:Float):Void
	{
		super.update(elapsed);

		var mapIndex = MapManager.getMapIndexFromScreenPosXY(FlxG.mouse.x, FlxG.mouse.y);
		var canBuild = MapManager.canBuildHere(mapIndex, roomType);

		// Show this sprite if we are on a tile where we can build the room
		visible = canBuild;

		if (canBuild)
		{
			var pos = MapManager.getScreenXYfromMapIndex(mapIndex);
			x = pos.x;
			y = pos.y;

			// We can build the room here. If LMB is pressed, we will do so
			if (FlxG.mouse.pressed)
			{
				var roomId = RoomManager.buildNewRoom(mapIndex, roomType);
				MapManager.setMapTilesToRoom(mapIndex, roomType, roomId);
				StateManager.setGameState(Game_Run);
			}
		}
	}
}
