package classes;

import enums.RoomSizeType;
import flixel.FlxG;
import managers.ConfigManager;
import managers.MapManager;
import managers.RoomManager;
import managers.StateManager;

class DeleteRoomFrame extends AtlasSprite
{
	var roomSize:RoomSizeType = RoomSizeType.None;

	public function new()
	{
		super();
	}

	override public function update(elapsed:Float):Void
	{
		super.update(elapsed);

		var mapIndex = MapManager.getMapIndexFromScreenPosXY(FlxG.mouse.x, FlxG.mouse.y);

		// Check if a room is present on this map tile
		if (MapManager.isValidMapIndex(mapIndex))
			if (MapManager.isOccupied(mapIndex))
			{
				var roomId = MapManager.getRoomIdFromTile(mapIndex);
				var firstMapIndex = MapManager.getFirstRoomIndex(roomId);
				var roomType = RoomManager.getRoom(roomId).roomType;
				var roomSize = RoomManager.getRoomSizefromKind(roomType);
				setRoomSize(roomSize);
				var pos = MapManager.getScreenXYfromMapIndex(firstMapIndex);

				x = pos.x;
				y = pos.y;
				visible = true;

				if (FlxG.mouse.pressed)
				{
					RoomManager.removeRoom(roomId);
					MapManager.setMapTilesToRoom(firstMapIndex, roomType, 0); // Clearing map
					StateManager.setGameState(Game_Run);
				}
				return;
			}

		visible = false;
	}

	function setRoomSize(roomSize:RoomSizeType)
	{
		if (roomSize == this.roomSize)
			return;

		switch (roomSize)
		{
			case RoomSizeType.None:
				return;
			case RoomSizeType.S1x1:
				assignImageFromAtlas(ConfigManager.DELF1_GFX);
			case RoomSizeType.S2x1:
				assignImageFromAtlas(ConfigManager.DELF2_GFX);
			case RoomSizeType.S4x1:
				assignImageFromAtlas(ConfigManager.DELF4_GFX);
		}

		this.roomSize = roomSize;
	}
}
