package classes;

import managers.SoundManager;
import flixel.group.FlxSpriteGroup;
import flixel.text.FlxText;
import flixel.util.FlxColor;
import managers.ConfigManager;
import managers.EventManager;
import managers.StateManager;
import managers.WarehouseManager;

class Hud extends FlxSpriteGroup
{
	var aboutButton:TTButton;
	var soundButton:TTButton;
	var buildButton:TTButton;
	var deleteButton:TTButton;

	var toolsSign:TTSprite;
	var rocketsSign:TTSprite;
	var timeSign:TTSprite;

	var toolsText:FlxText;
	var rocketsText:FlxText;
	var timeText:FlxText;

	public function new()
	{
		super();

		// First line: Buttons
		aboutButton = new TTButton(ConfigManager.ABOUTBUTTON_GFX, onClickAboutButton, "", "About...");
		aboutButton.setPosition(20 + 0 * 120, 20);
		add(aboutButton);

		soundButton = new TTButton(ConfigManager.SOUNDBUTTONON_GFX, onClickSoundButton, "", "Sound on/off");
		soundButton.setPosition(20 + 1 * 120, 20);
		add(soundButton);

		buildButton = new TTButton(ConfigManager.BUILDBUTTON_GFX, onClickBuildButton, "", "Build...");
		buildButton.setPosition(20 + 2 * 120, 20);
		add(buildButton);

		deleteButton = new TTButton(ConfigManager.DELETEBUTTON_GFX, onClickDeleteButton, "", "Delete...");
		deleteButton.setPosition(20 + 3 * 120, 20);
		add(deleteButton);

		// Second line: Signs and counters
		toolsSign = new TTSprite(ConfigManager.HUD_TOOLS, "Tools");
		toolsSign.setPosition(20 + 0 * 160, 100);
		add(toolsSign);

		rocketsSign = new TTSprite(ConfigManager.HUD_ROCKETS, "Rockets");
		rocketsSign.setPosition(20 + 1 * 160, 100);
		add(rocketsSign);

		timeSign = new TTSprite(ConfigManager.HUD_TIME, "Time");
		timeSign.setPosition(20 + 2 * 160, 100);
		add(timeSign);

		toolsText = new FlxText(0, 0, 0, "   0", 22);
		toolsText.setFormat(ConfigManager.STDFONT_NORMAL, 22, FlxColor.BLACK);
		toolsText.setPosition(20 + 0 * 160 + 40, 100);
		add(toolsText);

		rocketsText = new FlxText(0, 0, 0, "   0", 22);
		rocketsText.setFormat(ConfigManager.STDFONT_NORMAL, 22, FlxColor.BLACK);
		rocketsText.setPosition(20 + 1 * 160 + 40, 100);
		add(rocketsText);

		timeText = new FlxText(0, 0, 0, "00:00:00", 22);
		timeText.setFormat(ConfigManager.STDFONT_NORMAL, 22, FlxColor.BLACK);
		timeText.setPosition(20 + 2 * 160 + 40, 100);
		add(timeText);

		EventManager.subscribe(TimeCountChanged, timeChanged);
		EventManager.subscribe(RocketsCountChanged, rocketCountChanged);
		EventManager.subscribe(ToolsCountChanged, toolsCountChanged);
		EventManager.subscribe(AllInvadersRemoved, allInvadersRemoved);
	}

	function timeChanged() // Callback
	{
		timeText.text = WarehouseManager.getElapsedTimeString();
	}

	function rocketCountChanged() // Callback
	{
		rocketsText.text = WarehouseManager.getRocketCountString();
		rocketsSign.popOut();
		SoundManager.playSound(ConfigManager.COUNTER_SOUND);
	}

	function toolsCountChanged() // Callback
	{
		toolsText.text = WarehouseManager.getToolCountString();
		toolsSign.popOut();
		SoundManager.playSound(ConfigManager.COUNTER_SOUND);
	}

	function allInvadersRemoved() // Callback
	{
		// Not updating the time info after all invaders have been removed
		EventManager.unsubscribe(TimeCountChanged, timeChanged);
		EventManager.unsubscribe(AllInvadersRemoved, allInvadersRemoved);
	}

	function onClickAboutButton()
	{
		StateManager.setGameState(About);
	}

	function onClickSoundButton()
	{
		SoundManager.toggleSound();

		if (SoundManager.soundOn)
			soundButton.assignSprites(ConfigManager.SOUNDBUTTONON_GFX);
		else
			soundButton.assignSprites(ConfigManager.SOUNDBUTTONOFF_GFX);

		EventManager.trigger(SoundSettingChanged);
	}

	function onClickBuildButton()
	{
		StateManager.setGameState(Select);
	}

	function onClickDeleteButton()
	{
		StateManager.setGameState(Remove);
	}
}
