package classes;

import managers.ConfigManager;
import enums.InvaderState;

class Invader extends AtlasSprite
{
	static final MOVE_SPAN:Float = 50; // Pixels left/right to move from starting pos
	static final INVADER_SPEED_X:Float = 10;

	var state:InvaderState = None;
	var startingPosX:Float;
	var leftBoundaryX:Float;
	var rightBoundaryX:Float;

	public function new()
	{
		super();
		assignAtlas();
		animation.addByIndices(Std.string(InvaderState.Normal), ConfigManager.INVADER_GFX, [0, 1, 2, 3, 4, 5, 4, 3, 2, 1], ".png", 4, true);
		setState(Normal);
		resetSizeFromFrame();		

		// Adjusting hitbox
		this.setSize(width - 4, height - 4);
		centerOrigin();
		this.centerOffsets();
	}

	public function setStartingPosition(x:Float, y:Float, moveDir:Int)
	{
		startingPosX = this.x = x;
		this.y = y;

		leftBoundaryX = startingPosX - MOVE_SPAN;
		rightBoundaryX = startingPosX + MOVE_SPAN;

		if (moveDir < 0)
			velocity.x = -INVADER_SPEED_X;
		else
			velocity.x = INVADER_SPEED_X;
	}

	public function setState(newState:InvaderState)
	{
		if (state == newState)
			return;

		if (newState != None)
			animation.play(Std.string(newState));

		state = newState;
	}

	public override function update(elapsed:Float)
	{
		super.update(elapsed);

		if (x <= leftBoundaryX)
		{
			x = leftBoundaryX;
			velocity.x = INVADER_SPEED_X;
		}
		else if (x >= rightBoundaryX)
		{
			x = rightBoundaryX;
			velocity.x = -INVADER_SPEED_X;
		}
	}
}
