package classes;

import haxe.exceptions.NotImplementedException;
import managers.PathManager;
import helpers.MapIndex;
import enums.PathCalcState;

// This is the workhorse for pathfinding. It shall be created per entity.
// Processing happens in steps on each update() call, to simulate some 
// sort of 'threading'. It usually finds a path, but not the shortest one.
// Not at all the shortest one.
class PathFinder
{
    public var state(default, null):PathCalcState = None;
	public var path(default, null):Array<MapIndex>;

	var piStart:MapIndex;
	var piEnd:MapIndex;
	var openList:Array<Array<MapIndex>>;
	var doneList:Array<MapIndex>;
    
    public function new()
    {
		path = new Array<MapIndex>();
    }

	public function startPathCalc(miStart:MapIndex, miEnd:MapIndex)
	{
		state = Running;
		path = null;

		piStart = PathManager.mapIndexToPathIndex(miStart);
		piEnd = PathManager.mapIndexToPathIndex(miEnd);

		openList = new Array<Array<MapIndex>>();
		doneList = new Array<MapIndex>();

		openList.push([piStart]);
		doneList.push(piStart);
	}

	public function update():PathCalcState
	{
		if (state == Running) 
        {
			var checkPath = openList.shift(); // Get one path from list and search further
			if (checkPath == null)
			{
				path = new Array<MapIndex>(); // No path found. Here, this should only happen when start==end
				state = Finished;
			}
			else
			{
				var lastOfPath = checkPath[checkPath.length - 1];
				var options = PathManager.getAdjacendWalkable(lastOfPath);
				for (pi in options)
				{
					if (pi.isValueIn(doneList) == false)
					{
						var newPath = checkPath.concat([pi]);

						if (pi != piEnd)
						{
							openList.push(newPath);
							doneList.push(pi);
						}
						else
						{
							path = newPath.map(item -> PathManager.pathIndexToMapIndex(item));
							state = Finished;
						}
					}
				}
			}
        }

		return state;
	}
}