package classes;

import flixel.math.FlxPoint;
import managers.SoundManager;
import enums.RocketState;
import flixel.FlxG;
import managers.ConfigManager;
import managers.ExplosionManager;
import managers.InvaderManager;

class Rocket extends AtlasSprite
{
	static final ROCKET_SPEED_X = 30.0;

	var rocketState:RocketState = None;
	var targetPosY = 0.0;

	public function new()
	{
		super();
		assignAtlas();
		animation.addByIndices(Std.string(RocketState.Normal), ConfigManager.ROCKET_GFX, [0], ".png", 0, false);
		animation.addByIndices(Std.string(RocketState.Flying), ConfigManager.ROCKET_GFX, [0, 1, 2], ".png", 10, true);
		animation.play(Std.string(RocketState.Normal)); // needed so that resetSizeFromFrame works
		resetSizeFromFrame();

		// Adjusting hitbox
		this.setSize(width - 4, height - 12);
		centerOrigin();
		this.offset = new FlxPoint(2, 0); 
	}

	public function setState(newState:RocketState)
	{
		if (rocketState == newState)
			return;

		if (newState != None)
			animation.play(Std.string(newState));

		rocketState = newState;
	}

	public function moveTo(targetPosY:Float)
	{
		if (rocketState == Flying)
			return;

		this.targetPosY = targetPosY;
		setState(Flying);
	}

	public override function update(elapsed:Float)
	{
		super.update(elapsed);

		switch (rocketState)
		{
			case None:
				setState(Flying);
			case Normal:
			case Flying:
				handleMovement(elapsed);
				checkCollision();
		}
	}

	function handleMovement(elapsed:Float)
	{
		var deltaPos = targetPosY - y;
		var deltaPosAbs = Math.abs(deltaPos);
		var moveDist = elapsed * ROCKET_SPEED_X;

		if (deltaPosAbs <= moveDist)
		{
			this.kill();
			ExplosionManager.startExplosion(x - 7, y, ConfigManager.EXPL_ROCKET_GFX);
			SoundManager.playSound(ConfigManager.FAILED_SOUND); // Rocket didn't hit anything
		}
		else
		{
			y += (deltaPos / deltaPosAbs * moveDist);
		}
	}

	function checkCollision()
	{
		FlxG.overlap(this, InvaderManager.getLayer(), handleCollision);
	}

	function handleCollision(rocket:Rocket, invader:Invader)
	{
		InvaderManager.hitInvader(invader.ID);
		this.kill();
	}
}
