package classes;

import managers.WarehouseManager;
import flixel.group.FlxSpriteGroup.FlxTypedSpriteGroup;
import enums.RoomState;
import enums.RoomType;
import managers.ConfigManager;

class Room extends FlxTypedSpriteGroup<AtlasSprite>
{
	public var roomType(default, null):RoomType;
	
	var roomState:RoomState = None;
	var progressPercent:Float = 0.0; 
	
	var roomSprite:AtlasSprite;
	var bar_bg:AtlasSprite;
	var bar_fg:AtlasSprite;

	public function new(roomType:RoomType)
	{
		super();

		this.roomType = roomType;

		if (roomType == RoomType.None)
			return;

		// Animated room sprite
		roomSprite = new AtlasSprite();
		roomSprite.assignAtlas();

		roomSprite.animation.addByIndices(Std.string(RoomState.Active), ConfigManager.ROOM_GFX[roomType], [0, 1, 2, 3], ".png", 2, true);
		roomSprite.animation.addByIndices(Std.string(RoomState.Waiting), ConfigManager.ROOM_GFX[roomType], [4], ".png", 0, false);

		roomSprite.animation.play(Std.string(RoomState.Waiting)); // needed so that resetSizeFromFrame works
		roomSprite.resetSizeFromFrame();
		roomSprite.centerOrigin();
		add(roomSprite);

		// Production progress bar - not used for CannonControl. Todo: Better set a flag for this on creation
		if (roomType != RoomType.Room_CannonControl)
			createProgressbar();
	}

	public override function update(elapsed:Float)
	{
		super.update(elapsed);

		switch (roomState)
		{
			case None:
				setState(Waiting);
			case Waiting:
				checkProductionStart();
			case Active:
				handleProduction(elapsed);
		}	
	}

	function createProgressbar()
	{
		bar_bg = new AtlasSprite();
		bar_fg = new AtlasSprite();

		bar_bg.assignImageFromAtlas(ConfigManager.PROGRESSBAR_BG_GFX);
		bar_fg.assignImageFromAtlas(ConfigManager.PROGRESSBAR_FG_GFX);

		bar_bg.origin.set(0, 0); // upper left corner
		bar_fg.origin.set(0, 0);

		bar_bg.setPosition(2, roomSprite.height - bar_bg.height - 2);
		bar_fg.setPosition(2, roomSprite.height - bar_fg.height - 2);

		bar_bg.alpha = 0.5;
		bar_fg.alpha = 0.5;

		bar_bg.scale.x = (roomSprite.width - 4) / bar_bg.width;
		setProgressValue(0.0);

		add(bar_bg);
		add(bar_fg);
	}	

	public function setProgressValue(percent:Float)
	{
		if ((bar_bg == null) || (bar_fg == null))
			return;

		progressPercent = percent;
		bar_fg.scale.x = ((roomSprite.width - 4) / bar_fg.width) * percent / 100.0;
	}

	public function setState(newState:RoomState)
	{
		if (roomState == newState)
			return;

		if (newState != None)
			roomSprite.animation.play(Std.string(newState));

		roomState = newState;
	}

	function checkProductionStart()
	{
		var neededGoodsType = ConfigManager.ROOM_NEED_TYPE[roomType];
		var neededGoodsAmount = ConfigManager.ROOM_NEED_QUANTITY[roomType];
		var availableGoodsAmount = WarehouseManager.getWareCount(neededGoodsType);

		if (availableGoodsAmount >= neededGoodsAmount) // Can start operation
		{			
			WarehouseManager.addWareCount(neededGoodsType, -neededGoodsAmount);
			setProgressValue(0);
			setState(Active);
		}
	}
	
	function handleProduction(elapsed:Float)
	{
		if (ConfigManager.ROOM_TIME[roomType] == 0) // Constantly running
			return;

		var progress = progressPercent + elapsed / ConfigManager.ROOM_TIME[roomType] * 100.0;
		
		if (progress <= 100.0) // Not finished yet
		{
			setProgressValue(progress);
		}
		else
		{
			var producedGoodsType = ConfigManager.ROOM_PROD[roomType];
			var producedGoodsAmount = 1; // Different values might be stored to ConfigManager later 
			
			WarehouseManager.addWareCount(producedGoodsType, producedGoodsAmount);
			setProgressValue(0);
			setState(Waiting);
		}
	}
}
