package classes;

import flixel.FlxG;
import flixel.group.FlxSpriteGroup;
import flixel.text.FlxText;
import flixel.util.FlxColor;
import managers.ConfigManager;

// Provides a single image with simple tooltip-like functionality
// on mouseOver and a 'pop-out' effect on trigger
class TTSprite extends FlxSpriteGroup
{
	var image:AtlasSprite;
	var tooltip:FlxText;

	public function new(spriteName:String, toolText:String)
	{
		super();
		image = new AtlasSprite();
		image.assignImageFromAtlas(spriteName);
		add(image);

		tooltip = new FlxText(0, 0, 0, toolText, 18);
		tooltip.setFormat(ConfigManager.STDFONT_NORMAL, 18, FlxColor.BLACK);
		tooltip.setPosition(x + (image.width - tooltip.width) / 2, y + image.height - 5);
		tooltip.visible = false;
		add(tooltip);
	}

	public function popOut()
	{
		image.scale.set(1.5, 1.5);
	}

	override public function update(elapsed:Float):Void
	{
		super.update(elapsed);
		showTooltip(FlxG.mouse.overlaps(image));
		checkScaling(elapsed);
	}

	function showTooltip(showIt:Bool)
	{
		if (showIt)
			tooltip.visible = true;
		else
			tooltip.visible = false;
	}

	function checkScaling(elapsed:Float)
	{
		if (image.scale.x == 1.0) return;

		var newScale = image.scale.x - elapsed;
		newScale = newScale < 1.0 ? 1.0 : newScale;
		image.scale.set(newScale, newScale);
	}
}
