package helpers;

import enums.Direction;

// My own implemention of a 'Vector2Int' thing. As an abstract, to
// cover operator overloading. 'this' is an array of 2 ints.
// See https://riptutorial.com/haxe/example/16407/operator-overloading
abstract MapIndex(Array<Int>)
{
	public inline function new(x:Int = 0, y:Int = 0)
	{
		this = [x, y];
	}

	public inline function getX():Int
	{
		return this[0];
	}

	public inline function getY():Int
	{
		return this[1];
	}

	@:op(A == B)
	public static function equals<A, B>(A:MapIndex, B:MapIndex):Bool
	{
		trace("== called MapIndex" + A + "/" + B);
		return A.getX() == B.getX() && A.getY() == B.getY();
	}

	@:op(A != B)
	public static function notequal<A, B>(A:MapIndex,  B:MapIndex):Bool
	{
		return A.getX() != B.getX() || A.getY() != B.getY();
	}

	@:op(A + B)
	public static function add(A:MapIndex, B:MapIndex):MapIndex
	{
		return new MapIndex(A.getX() + B.getX(), A.getY() + B.getY());
	}

	@:op(A - B)
	public static function substract(A:MapIndex,  B:MapIndex):MapIndex
	{
		return new MapIndex(A.getX() - B.getX(), A.getY() - B.getY());
	}

	// Used this, as standard 'Array<MapIndex>.contains()' did not
	// use the operator overloading as defined above
	public function isValueIn(array:Array<MapIndex>):Bool
	{
		for (e in array)
		{
			if ((e.getX() == getX()) && (e.getY() == getY()))
				return true;
		}

		return false;
	}

	public inline function getLeft():MapIndex
	{
		return new MapIndex(getX() - 1, getY());
	}

	public inline function getRight():MapIndex
	{
		return new MapIndex(getX() + 1, getY());
	}

	public inline function getUp():MapIndex
	{
		return new MapIndex(getX(), getY() - 1);
	}

	public inline function getDown():MapIndex
	{
		return new MapIndex(getX(), getY() + 1);
	}

	// Get other MapIndex in a certain walking direction
	public inline function getInDirection(dir:Direction):MapIndex
	{
		switch (dir)
		{
			case None:
				return new MapIndex(getX(), getY());
			case Left:
				return getLeft();
			case Right:
				return getRight();
			case Up:
				return getUp();
			case Down:
				return getDown();
		}
	}

	// Get direction of the other MapIndex (in relation to this one),
	// not considering diagonal options
	public inline function getDirectionOf(mi:MapIndex):Direction
	{
		if (mi.getX() - getX() > 0)
			return Right;
		else if (mi.getX() - getX() < 0)
			return Left;
		else if (mi.getY() - getY() > 0)
			return Down;
		else if (mi.getY() - getY() < 0)
			return Up;
		else 
			return None;
	}

	@:keep
	public inline function toString():String
	{
		return (getX() + "," + getY());
	}
}
