package managers;

import enums.EventType;

// Very basic event manager, where callbacks (void, void) can be registered and deregistered.
// Took some inspiration from https://kylemtravis.com/blog/haxe-signals/
class EventManager
{
	static var events:Map<EventType, Array<Dynamic>> = new Map<EventType, Array<Void->Void>>();

	public static function subscribe(event:EventType, callback:Void->Void)
	{
		if (events.exists(event) == false)
			events[event] = new Array<Void->Void>();

		events[event].push(callback);
	}

	public static function unsubscribe(event:EventType, callback:Void->Void)
	{
		if (events.exists(event) == false)
			return;

		events[event].remove(callback);
	}

	public static function trigger(event:EventType)
	{
		if (events.exists(event) == false)
			return;

		for (callback in events[event])
			callback();
	}

	public static function clear(?event:EventType)
	{
		if (event == null)
			events = new Map<EventType, Array<Void->Void>>();
		else
			events[event] = new Array<Void->Void>();
	}

	#if debug
	public static function debug_content()
	{
		var output:String = "Registered Events: ";

		for (event in Type.allEnums(EventType))
		{
			if (events.exists(event) == false)
			{
				events[event] = new Array<Void->Void>();
			}
			output += (Std.string(event) + "(" + Std.string(events[event].length) + ") ");
		}

		trace(output);
	}
	#end
}
