package managers;

import haxe.Timer;
import classes.Invader;
import flixel.group.FlxSpriteGroup;

class InvaderManager
{
	static final INVADERS_PER_LINE = 5; // 2 lines
	static final INVADER_Y_POS = [180, 240]; // invader row 0/1 fixed Y pos

	static var layer:FlxTypedSpriteGroup<Invader>;
	static var moveDir = [1.0, -1.0]; // Flag for movement direction row 0, 1
	static var invaderList = [
		new Array<Invader>(), // invader row 0
		new Array<Invader>() // invader row 1
	];

	public static function init(layer:FlxTypedSpriteGroup<Invader>)
	{
		InvaderManager.layer = layer;
		populateInvaders();
	}

	public static function getLayer()
	{
		return layer;
	}

	static function populateInvaders()
	{
		var invCounter = 0;

		for (row in 0...2)
			for (ii in 0...INVADERS_PER_LINE)
			{
				var inv = new Invader();
				inv.setStartingPosition(80 + ii * 80 - 27, INVADER_Y_POS[row], row * -1);
				invaderList[row].push(inv);
				layer.add(inv);
			}
	}

	public static function hitInvader(id:Int)
	{
		var inv = getInvaderById(id);
		ExplosionManager.startExplosion(inv.x, inv.y, ConfigManager.EXPL_INVADER_GFX);
		SoundManager.playSound(ConfigManager.EXPLODE_SOUND);
		inv.kill();

		if (checkAllInvadersGone() == true)
		{
			EventManager.trigger(AllInvadersRemoved);
			Timer.delay(function() { StateManager.setGameState(Win); }, 500); // Some delay to finish explosion
		}
	}

	static function getInvaderById(id:Int):Invader
	{
		for (row in 0...2)
			for (ii in 0...INVADERS_PER_LINE)
			{
				if (id == invaderList[row][ii].ID)
					return invaderList[row][ii];
			}

		return null; // Should not happen
	}

	static function checkAllInvadersGone():Bool 
	{
		for (row in 0...2)
			for (ii in 0...INVADERS_PER_LINE)
			{
				if (invaderList[row][ii].alive == true)
					return false;
			}

		return true;
	}
}
