package managers;

import classes.Room;
import enums.RoomSizeType;
import enums.RoomType;
import flixel.group.FlxSpriteGroup.FlxTypedSpriteGroup;
import helpers.MapIndex;

// Not utilizing the built-in kill/revive/recycle scheme (as with rockets and workers),
// but doing simple new/delete
class RoomManager
{	
	static final roomTypeSizes:Map<RoomType, RoomSizeType> = [
		None => None,
		Room_Tools => S2x1,
		Room_Rockets => S4x1,
		Room_CannonControl => S1x1
	];

	static final roomSizesX:Map<RoomSizeType, Int> = [None => 0, S1x1 => 1, S2x1 => 2, S4x1 => 4];

	static var layer:FlxTypedSpriteGroup<Room>;
	static var roomList:Map<Int, Room> = [];

	public static function init(layer:FlxTypedSpriteGroup<Room>)
	{
		RoomManager.layer = layer;
	}

	// Get the room size enum from the room type enum
	public static function getRoomSizefromKind(roomType:RoomType):RoomSizeType
	{
		return roomTypeSizes[roomType];
	}

	// Get the room tile size (X) from the room size enum
	public static function getRoomSizeXfromSize(roomSize:RoomSizeType):Int
	{
		return roomSizesX[roomSize];
	}

	// Get the room tile size (X) from the room type enum
	public static function getRoomSizeXfromKind(roomType:RoomType):Int
	{
		var roomSize:RoomSizeType = getRoomSizefromKind(roomType);
		return getRoomSizeXfromSize(roomSize);
	}

	public static function buildNewRoom(mapIndex:MapIndex, roomType:RoomType):Int
	{
		var newRoom = buildNewRoomXY(mapIndex.getX(), mapIndex.getY(), roomType);
		WorkerManager.addNewWorker(mapIndex);
		EventManager.trigger(RoomsChanged);
		SoundManager.playSound(ConfigManager.BUILD_SOUND);
		return newRoom;
	}

	static function buildNewRoomXY(map_x:Int, map_y:Int, roomType:RoomType):Int
	{
		// Create and position sprite, add to internal list
		var newRoom = new Room(roomType);
		var pixelPos = MapManager.getScreenXYfromMapIndexXY(map_x, map_y);
		newRoom.setPosition(pixelPos.x, pixelPos.y);
		layer.add(newRoom);
		roomList[newRoom.ID] = newRoom;
		return newRoom.ID;
	}

	public static function removeRoom(roomId:Int)
	{
		if (!isValidRoomId(roomId))
			return;

		WorkerManager.removeWorkers(1);

		var room = roomList[roomId];
		roomList.remove(roomId);
		room.kill();		
		layer.remove(room);		

		EventManager.trigger(RoomsChanged);
		SoundManager.playSound(ConfigManager.BUILD_SOUND);
	}

	public static function getRoom(roomId:Int):Room
	{
		return roomList.get(roomId);
	}

	public static function isValidRoomId(roomId:Int):Bool
	{
		return roomList.exists(roomId);
	}

	public static function getRoomCount(roomType:RoomType):Int
	{
		var count = 0;

		for (room in roomList)
		{
			if (room.roomType == roomType)
				count++;
		}

		return count;
	}
}
