package windows;

import flixel.math.FlxPoint;
import openfl.geom.Point;
import enums.RoomType;
import managers.SpriteManager;
import flixel.group.FlxSpriteGroup;
import managers.ConfigManager;
import classes.TTButton;
import flixel.FlxG;
import flixel.FlxSprite;
import flixel.text.FlxText;
import flixel.util.FlxColor;
import managers.StateManager;

class BuildWindow extends WindowBase
{
	static inline var SPACING = 24;
	static inline var START = 60;

	public function new()
	{
		super(10, 10, FlxG.width - 20, FlxG.height - 40, "Build a new Room", OnClickCloseButton);
	
		// Building a new Tools Workshop
		addBackgroundSection(20, 50, 420, 200);
		addRoomPicture(150, START + 0 * SPACING, Room_Tools);
		add(new FlxText(30, START + 4 * SPACING - 12, 400, "Tools Workshop: This room produces Tools, which can then be processed into rockets.", 18, true)).color = FlxColor.BLACK;
		addBuildButton(150, START + 6 * SPACING, "Build Tools", OnClickProdTools_Button);

		// Building a new Rockets Workshop
		addBackgroundSection(20, 265, 420, 225);
		addRoomPicture(70, START + 9 * SPACING, Room_Rockets);
		add(new FlxText(30, START + 13 * SPACING - 12, 400, "Rocket Workshop: This room produces Rockets from Tools. It will automatically start production when ressources are available.", 18, true)).color = FlxColor.BLACK;
		addBuildButton(150, START + 16 * SPACING, "Build Rockets", OnClickProdRockets_Button);

		// Building a new Cannon Control Room
		addBackgroundSection(20, 505, 420, 150);
		addRoomPicture(30, START + 19 * SPACING, Room_CannonControl);
		add(new FlxText(120, START + 19 * SPACING - 12, 320, "Cannon Control: This room speeds up the cannon movement. It does not use any goods and is always active.", 18, true)).color = FlxColor.BLACK;
		addBuildButton(150, START + 23 * SPACING, "Build Control", OnClickProdControl_Button);
		
		activateWindow(false);
	}

	function addBackgroundSection(x:Int, y:Float, width:Int, height:Int)
	{
		var bgSection = new FlxSprite(x, y);
		bgSection.makeGraphic(width, height, 0xFFD0D0D0);
		add(bgSection);
	}

	function addRoomPicture(x:Float, y:Float, roomType:RoomType)
	{
		var pic = SpriteManager.getImage(ConfigManager.ROOM_GFX[roomType] + "0");
		pic.scale = new FlxPoint(2, 2);
		pic.origin = new FlxPoint(0, 0);
		pic.setPosition(x, y);
		add(pic);
	}

	function addBuildButton(x:Float, y:Float, text:String, callback:()->Void)
	{
		var button = new TTButton(ConfigManager.WIDEBUTTON_GFX, callback, text, "");
		button.setPosition(x, y);
		add(button);
	}

	function OnClickCloseButton():Void
	{
		activateWindow(false);
		StateManager.setGameState(Game_Run);
	}

	function OnClickProdTools_Button():Void
	{
		activateWindow(false);
		PlayState.instance.createRoomBuildFrame(Room_Tools);
		StateManager.setGameState(Place);
	}

	function OnClickProdRockets_Button():Void
	{
		activateWindow(false);
		PlayState.instance.createRoomBuildFrame(Room_Rockets);
		StateManager.setGameState(Place);
	}

	function OnClickProdControl_Button():Void
	{
		activateWindow(false);
		PlayState.instance.createRoomBuildFrame(Room_CannonControl);
		StateManager.setGameState(Place);
	}
}
